<?php
define('DEBUG',1);
if(!empty($_SERVER['DOCUMENT_ROOT']))
{
	$path = rtrim ($_SERVER['DOCUMENT_ROOT'],'/');
}
else
{
	$path = '/usr/local/esia/public_html/';
	$_SERVER['DOCUMENT_ROOT'] = $path;
}
include $path.'/esia.conf.php';
//include_once $path.'/data/libs/CiscoSwitch.class.php';


//connection à la base de donnée
try
{
	$dbconnection = new PDO('pgsql:host='.DB_HOST.';port='.DB_PORT.';dbname='.DB_NAME, DB_USERNAME, DB_PWD);
	$dbconnection->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
}
catch(Exception $e)
{
	echo 'PDO SQL ERROR: '.$e->getMessage().'<br />';
	echo 'N° : '.$e->getCode();
}

include_once MVCMODEL;
include_once $path."/plugin/cloudswitch/model_switch.php";
include_once $path."/plugin/cloudswitch/libs/USL.class.php";
include_once $path."/plugin/cloudswitch/libs/cron.lib.php";
include_once $path."/data/libs/CiscoSwitch.class.php";

//préparation de la requête d'insertion
$queryInsert = $dbconnection->prepare("INSERT INTO esia_interface(interface_id,interface_name,interface_type,interface_device_id,interface_ifindex,interface_mac_addr)
		VALUES (DEFAULT,:name,:type,:dev_id,:ifindex,:mac);");

//prépartation de la requête d'update
$queryUpdate = $dbconnection->prepare("UPDATE esia_interface SET interface_name=:name,interface_type=:type,interface_mac_addr=:mac WHERE interface_device_id=:dev_id AND interface_ifindex=:ifindex;");

//prépartation de la requête d'update pour LLDP
$queryLLDPUpdate = $dbconnection->prepare("UPDATE esia_interface SET interface_name=:name,interface_type=:type WHERE interface_device_id=:dev_id AND interface_mac_addr=:mac;");
//préparation de la requête d'insertion pour LLDP
$queryLLDPInsert = $dbconnection->prepare("INSERT INTO esia_interface(interface_id,interface_name,interface_type,interface_device_id,interface_mac_addr)
		VALUES (DEFAULT,:name,:type,:dev_id,:mac);");

echo "Synchronisation des adresses mac des noeuds contenu dans la DB\n";
//on syncrhonise via SNMP les mac adresse
//provisionne le champ interface_mac_addr de la table interface
$nbrInsertInt=SyncMacAddressWithSNMP($dbconnection,$queryInsert,$queryUpdate);

//on syncrhonise via LLDP les mac adresse
//provisionne le champ interface_mac_addr de la table interface
//permet de lié par exemple un téléphone non snmp.
$nbrLLDP = SyncMacAddressWithLLDP($dbconnection, $queryLLDPInsert, $queryLLDPUpdate);
echo "Synchronisation TERMINEE: $nbrInsertInt nouvelles entrée via SNMP et $nbrLLDP via LLDP\n ";
//
//récupération de l'entiereté des switchs.
//

//Liaison via les adresse mac des liens non trunk. 
$list = Model_switch::getNodebyType($dbconnection,null,'switch');
//$list[] = Model_switch::InstanceSwitch($dbconnection, 'zyxel');
foreach ($list as $node)
{
    try
    {
        Model_switch::askScan($dbconnection,$node->getId());
        echo "SUCCESS:Start SCAN ".$node->getName()."\n";
    }
    catch(Exception $e)
    {
        echo "FAILED:Start SCAN ".$node->getName()."\n";
    }
}





/**
 * Synchronise les adresses MAC des interface réseau avec la base de donnée
 * Le noeud doit être compatible avec la if-mib
 * @param: $dbconnection l'objet de connexion PDO
 * @return: le nombre d'interface qui on été insérée. Ne retourne pas le nombre d'update
 * @exception: /
 */
function SyncMacAddressWithLLDP($dbconnection,$queryInsert,$queryUpdate)
{
	$querysearchByIP = $dbconnection->prepare('SELECT device_id FROM esia_device WHERE device_ip_addr=?;');
	$nbrInsertInt=0;
	$list = Model_switch::getNodebyType($dbconnection,null,'switch');
	foreach ($list as $node)
	{
		try
		{
			//on récucpère la liste des mac adresse connectée 
			$listmac = getLLDPMacAddressBySNMP($node);
			foreach($listmac as $lineMac)
			{
				$counter = $lineMac['counter'];
				$lldpid = $lineMac['lldpid'];
				$neighborid =$lineMac['neighborid'];
				$mac = $lineMac['mac'];
				//on va tenter de récupérer le nom de l'appareil et l'interface et de quand même essayer de les liée
				$value = $node->snmpget("1.0.8802.1.1.2.1.4.1.1.9.$counter.$lldpid.$neighborid");
				$value = formatString($value);
				//si le nom n'est pas vide on va tenter d'instancier un noeud
				if(!empty($value))
				{
					$distnode = null;
					try
					{
						//on instancie via le nom du noeud 
						$distnode = Model_esia::InstanceNode($dbconnection, $value);
					}
					catch(Exception $e )
					{
						//le noeud n'a pu etre instancier via le nom. Car il ne se trouve pas en DB 
						//on va tenter via l'adresse IP
						$results = $node->snmpwalkoid("1.0.8802.1.1.2.1.4.2.1.3.$counter.$lldpid.$neighborid");
						//l'oid lldpRemManAddrIfSubtype est composée de différent élément 
						//1.0.8802.1.1.2.1.4.2.1.3.{timer count},{port number}, {record number},{type}, {ip address of neighbor} . 
						$ipaddress=0;
						foreach($results as $oid => $value)
						{
							$value = formatInteger($value);
							//si value est égal à 3 c'est que l'addresse ip est contenue dans l'OID
							if($value == 3)
							{
								if(preg_match('#\.([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)$#', $oid,$reg))
								{
									$ipaddress = $reg[1];
									break;
								}
							}
						}
						if($ipaddress!=0)
						{
							$tmp =  new Device();
							$tmp->setIPv4Adress($ipaddress);
							$results = Model_esia::executeStatement($querysearchByIP,array($tmp->getIPv4DecimalAdress()));
							$row = $results->fetch(PDO::FETCH_OBJ);
							$distnode = Model_esia::InstanceNode($dbconnection, $row->device_id);
						}
						Trace("\tSYNC LLDP: ".$ipaddress.' SUCCESS');
					}
					//on regarde si on a reussi a instancier un noeud.
					if(isset($distnode))
					{
						Trace("\tSYNC LLDP: ".$distnode->getName().' Instance SUCCESS');
						//on élimine les switchs afin de ne pas pourrir avec la mac addresse virtuelle du switch et si SNMP est activer on a pu récupérer via la première méthode
						if($distnode->getType()!='switch' && empty($distnode->getSNMPVersion()))
						{
							//on récupère le nom de l'interface
							$value = $node->snmpget("1.0.8802.1.1.2.1.4.1.1.7..$counter.$lldpid.$neighborid");
							$name = formatString($value);
							//on a l'id du nom 
							$params = array(':name'=>$name,':type'=>6,':dev_id'=>$distnode->getId(),':mac'=>$mac );
							$result = Model_esia::executeStatement($queryUpdate,$params);
							if($result->rowCount()==0)
							{
								$result = Model_esia::executeStatement($queryInsert,$params);
								$nbrInsertInt++;
							}
							Trace("\tSYNC LLDP: ".$node->getId().' '.$node->getName().' INTERFACE: '."$mac == ".$result->rowCount());
						}
					}
				}
			}
		}
		catch (Exception $e)
		{
			Trace("\tSYNC LLDP: ".$node->getId().' '.$node->getName().' SNMP ERROR: '.$e->getMessage());
		}
	}
	return $nbrInsertInt;
}
/**
 * Synchronise les adresses MAC des interface réseau avec la base de donnée
 * Le noeud doit être compatible avec la if-mib 
 * @param: $dbconnection l'objet de connexion PDO
 * @param: $queryInsert requete préparée d'insertion des interface dans la DB
 * @param: $queryUpdate requete préparée d'update des interface dans la DB
 * @return: le nombre d'interface qui on été insérée. Ne retourne pas le nombre d'update
 * @exception: /
 */
function SyncMacAddressWithSNMP($dbconnection,$queryInsert,$queryUpdate)
{
	$nbrInsertInt=0;
	//mise à jour des interfaces réseau de chaque noeuds
	$nodes = Model_esia::getNodeList($dbconnection,null,null,Model_esia::NODE_INSTANCE);
	foreach($nodes as $node)
	{
		try
		{
			//	$node = new Device();
			//on vérife que SNMP est activé
			if(!empty($node->getSNMPVersion()))
			{
				Trace($node->getId().' '.$node->getName().' SYNC INTERFACE '. $node->getSNMPVersion());
				$results = $node->snmpwalkoid('1.3.6.1.2.1.2.2.1.6');
				foreach($results as $oid => $value)
				{
					try
					{
						if(preg_match('#Hex-STRING:\s+(.*)#', $value,$regs))
						{
							$mac=$regs[1];
							//on récupère le ifindex
							if(preg_match('#3\.6\.1\.2\.1\.2\.2\.1\.6\.([0-9]*)#', $oid,$regif))
							{
								$ifindex = $regif[1];
								//on prend le ifType et on ne prend que les interface de type 6
								$ifType = $node->snmpget('1.3.6.1.2.1.2.2.1.3.'.$ifindex);
								$ifType = preg_replace('#INTEGER:\s+#', '', $ifType);
								//if($ifType=="6"||$ifType=="117")
								//{
									//nous n'avons que les interfaces physiques.
									$name='';
									$resname= $node->snmpget('1.3.6.1.2.1.2.2.1.2.'.$ifindex);
									if(preg_match('#Hex-STRING:#', $resname))
									{
										$resname = preg_replace('#\s+#', '', $resname);
										$resname = preg_replace('#Hex-STRING:#', '', $resname);
										$name= Hex2String($resname);
									}
									elseif(preg_match('#STRING:\s+\"(.*)\"#', $resname,$regs))
									{
										$name= $regs[1];
									}
									$params = array(':name'=>$name,':ifindex'=>$ifindex,':type'=>$ifType,':dev_id'=>$node->getId(),':mac'=>$mac );
									$result = Model_esia::executeStatement($queryUpdate,$params);
									if($result->rowCount()==0)
									{
										$result = Model_esia::executeStatement($queryInsert,$params);
										$nbrInsertInt++;
									}
									Trace("\t".$node->getId().' '.$node->getName().' INTERFACE: '. $ifindex.' == '."$mac == ".$result->rowCount());
								//}
							}
						}
					}
					catch (Exception $e)
					{
						Trace("\t".$node->getId().' '.$node->getName().' INTERFACE: '.$e->getMessage());
					}
				}
			}
		}
		catch (Exception $e)
		{
			Trace("\t".$node->getId().' '.$node->getName().' SNMP ERROR: '.$e->getMessage());
		}
	}
	return $nbrInsertInt;
}
?>
