<?php
/**
 * @name GenConf
 * @description Classe de base pour l'USL, elle se charge de parser les XML et de remplacer les paramètre
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.00
 * @date 29/04/2013
 */
class GenConf {
	private $document_xml;
	private $xmlfile;
	private $conf;
	private $modeconf;
	/*@NAME: __construct
	 *@IN: $tplpath chemin d'accès au fichier template
	*@OUT: /
	*@TREAT: Constructeur par défaut de la classe GenConf
	*@EXCEPTION: 	GenConfException si $tplpath est vide
	*/
	public function __construct($tplpath)
	{
		if(empty($tplpath)||!isset($tplpath))
			throw new GenConfException("File no found");
		$this->conf='';
		$this->document_xml = new DomDocument();
		$this->document_xml->load($tplpath);
	}
	/*@NAME: confmode
	 *@IN: le mode de configuration qu'on souhaite 
	 *		configuration, configuration de base
	 *		interface $1 ,il faut alors spécifier un paramètre suplémentaire qui est le nom de l'interface
	*@OUT: /
	*@TREAT: Se positionne dans l'XML sur le mode de configuration demandé
	*@EXCEPTION:
	*/	
	public function confmode()
	{
		$this->modeconf=func_get_arg (0);
		$elements = $this->document_xml->getElementsByTagName("$this->modeconf");
		foreach($elements as $element)
		{
			$enf = $element->childNodes;
			foreach($enf as $e)
			{
				if($e->nodeName==$this->modeconf )
				{
					$value=$e->nodeValue;
					$nbr=preg_match_all('#\$[1-9]#',$value,$regs);
					if($nbr>=1)
					{
						for($i=1;$i<=$nbr;$i++)
						{
							$x =func_get_arg($i);
							$value=str_replace("$$i",$x,$value);
							//$patern='#\$'.$i.'#';
						}
					}
					$this->conf.=$value."\n";
					break;
				}
			}
		}
	}
	/*@NAME: exitmode
	 *@IN: /
	*@OUT: /
	*@TREAT: Quitte le mode de configuration est charge la commande "exit" si nécessaire
	*@EXCEPTION:
	*/
	public function exitmode()
	{
		$elements = $this->document_xml->getElementsByTagName("$this->modeconf");
		foreach($elements as $element)
		{
			$enf = $element->childNodes;
			foreach($enf as $e)
			{
				if($e->nodeName=="exit")
				{
					$this->conf.=$e->nodeValue."\n";
					break;
				}
			}
		}
	}
	/*@NAME: getcommand
	 *@IN: la commande demandée USL par ex: "show version"
	*@OUT: la commande demandée pour le fabricant
	*@TREAT: parse l'xml et retourne la commande demandée 
	*		attention les $* ne sont pas remplacé
	*@EXCEPTION:
	*/	
	public function getcommand()
	{
		$numargs = func_num_args();
		$x=func_get_arg (0);
		$flag="0";
		//echo "$x <br />";
		if(preg_match("#no\s(.*)#",$x,$regs))
		{
			//echo 'tr no <br />';
			$elements = $this->document_xml->getElementsByTagName("no");
			foreach($elements as $element)
			{
				//echo $element->nodeName."<br />";
				if($element->nodeName=="no")
				{
					$flag=$element->nodeValue." ";
					//	echo "flag: $flag <br />";
					$x=$regs[1];
				}
			}
		}
		if(preg_match_all("#([0-9a-zA-Z]*)\s([0-9a-zA-Z]*)#",$x,$regs))
		{
			$regs = explode(" ",$x);
			$elements = $this->document_xml;
			$element=$elements;
			$nbrelem =  count ($regs)-1;
			for($i=0;$i < $nbrelem;$i++)
			{
				//echo "regs: $regs[$i] <br />";
				if($i==0)
					$element  = $element->getElementsByTagName($this->modeconf);
				else
					$element  = $element->getElementsByTagName($regs[$i]);
				$element = $this->parsageNode($element,$regs[$i]);
				//echo $element->nodeName."<br />";
			}
			$x=$regs[$nbrelem];
			$arbre=$element;
		}
		else
		{
			//echo "NO REGEX $this->modeconf<br />";
			$elements = $this->document_xml->getElementsByTagName($this->modeconf);
			foreach($elements as $element)
			{
				//echo "reg $element->nodeName <br />";
				if($element->parentNode->nodeName=="configuration")
					$arbre=$element;
				elseif($element->nodeName=="configuration")
					$arbre=$element;
			}
			//echo "reg $arbre->nodeName <br />";
			//$arbre = $elements->item(0);
		}
		//echo $arbre->nodeName."<br />";
		if($flag!="0")
			$value = $this->parsage($arbre,$x,$flag);
		else
			$value = $this->parsage($arbre,$x);
		return $value;
	}
	/*@NAME: confmode
	 *@IN: paramètres variables: le premier est la commande USL demandée et ensuite les paramètres propre au sw
	*		les paramètres remplaceront les $1,$2,$3,etc
	*@OUT: la commande pour le switch
	*@TREAT: parse l'xml et retourne la commande demandée et remplace également les paramètres $*
	*@EXCEPTION: 	
	*/
	public function command()
	{
		$numargs = func_num_args();
		$x=func_get_arg(0);
		$flag="0";
		$value =$this->getcommand(func_get_args());
		//on va remplacer les $1,$2, etc par les argument passé en paramètre
		$patern='#(\$[1-9]*)#';
		$nbr=preg_match_all($patern,$value,$regs);
		if($nbr>=1)
		{
			//Evite un soucis quand on a une chaine de type $1 $2 $1
			//avant il cherchait un argument supplémentaire
			$arrtm = array();
			foreach($regs[1] as $arg) 
			{
				if(!in_array($arg,$arrtm))
					$arrtm[]= $arg;
			}
			$i=1;
			//on parcours le tableau de remplacement et on change toute les occurences
			foreach($arrtm as $argre)
			{
				$x =func_get_arg($i);
				//echo "replace $$i == $x\n";
				$value=str_replace("$$i",$x,$value);
				$i++;
			}
		}
		// si il y a plusieurs espace on les remplaces par 1 seuls
		$value=preg_replace('#[[:blank:]]+#',' ',$value);
		// si il y a un retour de chariot suivit de plusieurs espace on les remplaces par 1 seuls retour de chariots
		$value=preg_replace('#\n[[:blank:]]+#',"\n",$value);
		$this->conf.=$value."\n";
	//	echo 'value: '.$value.'<br />';
	}
	/*@NAME: writecommand
	 *@IN: $com la commande du fabricant
	*@OUT: /
	*@TREAT: ajoute à la configuration actuelle la commande entrée en paramètre
	*@EXCEPTION:
	*/
	public function writecommand($com)
	{
		$this->conf.=$com;
	}
	/*@NAME: getConfiguration
	 *@IN: /
	*@OUT: la configuration générée
	*@TREAT: retourne la configuration générée
	*@EXCEPTION:
	*/
	public function getConfiguration()
	{
		$this->conf=preg_replace('#\n+#',"\n",$this->conf);
		$this->conf=preg_replace('# +#'," ",$this->conf);
		$this->conf=rtrim($this->conf);
		return $this->conf."\n";
	}
	/*@NAME: getHTMLConfiguration
	 *@IN: /
	*@OUT: la configuration générée au format HTML
	*@TREAT: retourne la configuration générée  au format HTML
	*@EXCEPTION:
	*/
	public function getHTMLConfiguration()
	{
		$this->conf=$this->getConfiguration();
		return str_replace("\n",'<br />',$this->conf);
	}
	/*@DEPRECATED
	 * @NAME: affiche
	 *@IN: /
	*@OUT: la configuration générée au format HTML
	*@TREAT: retourne la configuration générée  au format HTML
	*@EXCEPTION:
	*/
	public function affiche()
	{
		return "$this->company $this->device $this->os	$this->version <br />file: $this->xmlfile";
	}
	/*@NAME: parsage
	*@IN: $noeud le noeud DOM qu'on veut parser
	*		$name le nom du noeud qu'on souhaite trouver
	*		$flag (optionnelle) contient la commande pour annuler la commande du noeud (par ex: no  shutdown)
	*@OUT: la valeur pour le noeud donné
	*@TREAT: parse l'entièreté de l'XML pour recherche le noeud donné
	*		Fonction récursive
	*@EXCEPTION:
	*/
	private function parsage($noeud,$name,$flag="0")// Fonction de parsage d'enfants
	{
		$value='';
		//echo "par : ".$name;
			$enfants_niv1 = $noeud->childNodes; // Les enfants du noeud traité
			//echo 'parsageniv1'.$noeud->nodeName."<br />";
			foreach($enfants_niv1 as $enfant) // Pour chaque enfant, on vérifie...
			{
				//echo "parsage $enfant->nodeName == $name<br />";
				if($enfant->nodeName=="$name")
				{
					if($flag=="0")
						$value.= $enfant->nodeValue."\n";
					else
						$value.= $flag." ".$enfant->nodeValue."\n";
				}
				else
				{
					if($enfant->hasChildNodes()) // ... s'il a lui-même des enfants
					{
						//echo "recur".$enfant->nodeName."<br />";
						$value.= $this->parsage($enfant,$name,$flag);
					}
				}
			}
		return  $value;
	}
	/*@NAME: parsageNode
	 *@IN: $noeud le noeud DOM qu'on veut parser
	*		$name le nom du noeud qu'on souhaite trouver
	*@OUT: la valeur pour le noeud donné
	*@TREAT: parse l'entièreté de l'XML pour recherche le noeud donné
	*		Fonction récursive
	*@EXCEPTION:
	*/	
	private function parsageNode($noeud,$name)// Fonction de parsage d'enfants
	{
		$value='';
		//echo $noeud->nodeName."<br /><br />";
		// $enfants_niv1 = $noeud->childNodes; // Les enfants du noeud traité
		foreach($noeud as $enfant) // Pour chaque enfant, on vérifie...
		{
		//	echo "noed $name:".$enfant->nodeName."<br />";
			if($enfant->nodeName=="$name")
			{
		//		echo '$name ==>'.$enfant->nodeName."<br />";
				return $enfant;
			}
			else
			{
				if($enfant->hasChildNodes()) // ... s'il a lui-même des enfants
				{
					$enfant= $enfant->childNodes;
					//echo "recur".$enfant->nodeName."<br />";
					$enfant= $this->parsageNode($enfant,$name);
					if(is_object($enfant) && $enfant->nodeName=="$name")
					{
						//echo $enfant->nodeName."<br />";
						return $enfant;
					}
				}
			}
		}
	}
}
class GenConfException extends Exception
{
}
?>
