<?php
/**
 * @name RemoteSwitch.class.php
* @description Classe RemoteSwitch du plugin "switch" du projet ESIA, connexion via une GESA
* @author Nicolas Biersart <n.biersart@esia-network.com>
* @contributor
* @copyright Nicolas Biersart 2012
* @license
* @esia 1.00
* @version 1.0.0
* @date 17/06/2015
*/

include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once LIB_SNMP;
include_once 'switch.class.php';
include_once TELNET_CLASS;
include_once SSH_CLASS;

class RemoteSwitch extends RemoteDevice implements switchInterface
{
	const GESA_MIB_INTERFACE = 'get_snmp_sw_interface2';
	const GESA_MIB_INTERFACE_STAT = 'get_snmp_sw_interface_stat';
	/*@NAME: __construct
	 *@IN: /
	 *@OUT: /
	 *@TREAT: constructeur de base
	 *@EXCEPTION: /
	 */
	public function __construct($dev=null,$gesa=NULL)
	{
		if(get_class($dev)=='Device')
		{
			parent::constructDevice($dev);
		}
		elseif(get_class($dev)=='RemoteDevice')
		{
			parent::constructDevice($dev);
			$this->gesa = $gesa;
		}
		else
		{
			parent::__construct();
		}
		$this->telnetclass='Telnet';
		$this->sshclass='ssh';
		$this->list_interface=array();
		$this->interface_class='interfaces';
	}
	/*@NAME: setAllInterfaceBySNMP
	 *@IN: /
	 *@OUT: /
	 *@TREAT: récupère en SNMP la liste de toutes les interfaces et instancie les objets correspondant
	 *@EXCEPTION:  DeviceSNMPException si une erreur survient
	 */
	public function setAllInterfaceBySNMP()
	{
		return $this->getInterfacesMib();
	}
	public function getInterfacesMib($all=true)
	{
		try
		{
			//echo 'ici';
			//$param = $this->getSNMPCommunity();
			//$interfaces = $this->gesa->check_nrpe($this,RemoteSwitch::GESA_MIB_INTERFACE ,$param);
			$line=0;
			$data=array();
			$i=0;
			do
			{
				$param = $this->getSNMPCommunity()." ".$line;
				$arr = $this->gesa->check_nrpe($this,RemoteSwitch::GESA_MIB_INTERFACE ,$param);
				$data=array_merge($data,$arr["data"]);
				$line = count($data);
				$i++;
			}
			while($arr["status"]=="MORE" && $i < 10);

			//array_pop($data);
			//$interfaces = $this->ThreadGESAError($interfaces, RemoteSwitch::GESA_MIB_INTERFACE);
			array_unshift($data,array('ifIndex','ifDescr','ifOperStatus','ifSpeed'));
			$interfaces = GESA::data_to_arrays($data);
			foreach ($interfaces as $line)
			{
				//echo print_r($line,true).'<br />';
				$inter = $this->instanceInterface($line);
				$this->interfaces[]=$inter;
			}
			return $this->interfaces;
		}
		catch(GesaConnectionFailed $e)
		{
			throw new DeviceException($e->getMessage());
		}
	}
	public function getInterfaceStat($ifindex,$all=true)
	{
		try
		{
			$param = $this->getSNMPCommunity().' '.$ifindex;
			$interfaces = $this->gesa->check_nrpe($this,RemoteSwitch::GESA_MIB_INTERFACE_STAT ,$param);

			$interfaces = $this->ThreadGESAError($interfaces, RemoteSwitch::GESA_MIB_INTERFACE_STAT);
			//$interfaces = GESA::data_to_arrays($interfaces);

			$inter = new interfaces();
			foreach ($interfaces as $line)
			{
				$inter->ifIndex=$line[0];
				$inter->ifDescr=$line[1];
				$inter->ifName=$line[2];
				$inter->ifOperStatus=$line[3];
				$inter->ifInOctets=$line[4];
				$inter->ifOutOctets=$line[5];
				$inter->ifInErrors=$line[6];
				$inter->ifOutErrors=$line[7];
				$inter->ifMtu=$line[8];
				$inter->ifSpeed=$line[9];
				$inter->ifLastChange=$line[10];
				$inter->ifAlias=$line[11];
			}
			return $inter;
		}
		catch(GesaConnectionFailed $e)
		{
			throw new DeviceException($e->getMessage());
		}
	}
	private function instanceInterface($line)
	{
		if(!isset($line['ifType']))
		{
			$line['ifType']=6;
		}
		$inter = new interfaces();
		$inter->ifIndex=$line['ifIndex'];
		$inter->ifDescr=$line['ifDescr'];
		$inter->ifType=$line['ifType'];
		$inter->ifMtu=$line['ifMtu'];
		$inter->ifPhysAddress=$line['ifPhysAddress'];
		$inter->ifAdminStatus=$line['ifAdminStatus'];
		$inter->ifOperStatus=$line['ifOperStatus'];
		$inter->ifSpeed=$line['ifSpeed'];
		return $inter;
	}
}
