<?php
/**
 * @name USL.class.php
* @description Classe Gérant l'USL du plugin switch
* @author Nicolas Biersart <n.biersart@esia-network.com>
* @contributor
* @copyright Nicolas Biersart 2012
* @license
* @esia 1.00
* @version 1.0.0
* @date 17/04/2013
*/
include_once 'GenSwitchConf.class.php';

class USL 
{
	protected $conf;
	protected $template_conf;
	/*@NAME: __construct
	 *@IN: fichiers vers le template de switch
	*@OUT: /
	*@TREAT: constructeur de base
	*@EXCEPTION: /
	*/
	public function __construct ($template)
	{
		try
		{
			$this->template_conf = $template;
			$this->conf = new GenConf($template);
			$this->conf->confmode('configuration');
		}
		catch(GenConfException $e)
		{
			throw new USLException('Bad File');
		}
	}
	/*@NAME: getConfigurationTemplate
	 *@IN: /$tpl
	*@OUT: le chemin d'accès au fichier de template de configuration
	*@TREAT: retourne le chemin d'accès au fichier de template de configuration
	*@EXCEPTION: /
	*/
	public function getConfigurationTemplate()
	{
		return $this->template_conf;
	}
	/*@NAME: setConfiguration
	 *@IN: /
	*@OUT: /
	*@TREAT: Active le mode de configuration du switch et charge le template
	*@EXCEPTION: USLException si le template n'est pas trouvé
	*/
	public function setConfiguration()
	{
		try
		{
			//echo $this->template_conf.'<br />';
			$this->conf = new GenConf($this->template_conf);
		}
		catch(GenConfException $e)
		{
			throw new USLException('unsuported manufacturer');
		}
	}
	/*@NAME: endConfiguration
	 *@IN: /
	*@OUT: /
	*@TREAT: Fini le mode de configuration
	*@EXCEPTION: /
	*/
	public function endConfiguration()
	{
		$this->conf->exitmode();
		$this->conf->confmode('configuration');
		$this->conf->command('save');
	}
	/*@NAME: getGenerateConfiguration
	 *@IN: /
	*@OUT: la configuration générée précédement
	*@TREAT: retourne la configuration générée
	*@EXCEPTION: /
	*/
	public function getGenerateConfiguration()
	{
		return $this->conf->getConfiguration();
	}
	/*@NAME: getGenerateConfiguration
	 *@IN: /
	*@OUT: la configuration générée précédement au format HTML
	*@TREAT: retourne la configuration générée au format HTML
	*@EXCEPTION: /
	*/
	public function getGenerateConfigurationHTML()
	{
		return $this->conf->getHTMLConfiguration();
	}
	/*@NAME: getUSL
	 *@IN: /
	*@OUT: l'objet de génération de commande
	*@TREAT: retourne l'objet de génération de commande
	*@EXCEPTION: /
	*/
	public function getUSL()
	{
		return $this->conf;
	}
	/*@NAME: ConfSwitch
	 *@IN: /
	*@OUT: /
	*@TREAT: passe en mode de configuration global
	*@EXCEPTION: /
	*/
	public function ConfSwitch()
	{
		$this->conf->confmode('global');
	}
	/*@NAME: ConfInterface
	 *@IN: Un objet interface
	*@OUT: /
	*@TREAT: Génére la configuration passé en paramètre par l'objet interface
	*@EXCEPTION: /
	*/
	public function ConfInterface($int)
	{
		$this->conf->confmode('configuration');
		if(is_object($int))
		{
			if($int->mode=='trunk')
			{
				$this->conf->command('no snmp macNotification');
				$this->conf->command('bridgeVlan',$int->name,$int->vlan);
			}
			elseif($int->mode=='static'||$int->mode=='untagged')
			{
				$this->conf->command('accessVlan',$int->name,$int->vlan);
			}
			elseif($int->mode=='dynamic')
			{
				$this->conf->command('dynamicVlan',$int->name,$int->vlan);
			}
		/*	if(!$int->getVoiceVlan())
			{
	
				if($int->getVoiceVlan()==1)
				{
					$this->conf->command('voiceVlan',$int->name,VOICE_VLAN);
				}
				elseif($int->getVoiceVlan()===0)
				{
					$this->conf->command('no voiceVlan',$int->name,VOICE_VLAN);
				}
			}*/
			$this->conf->confmode('interface',$int->name);
			if($int->description!==false && !empty($int->description))
			{
				$this->conf->command('description',$int->description);
			}
			if($int->reset==1)
			{
				$this->conf->command("reset");
			}
			if($int->ifAdminStatus=='up')
			{
				$this->conf->command("activate");
			}
			elseif($int->ifAdminStatus=='down')
			{
				//echo $int->getAdmStatus().'<br >';
				$this->conf->command("shutdown");
			}

			$this->conf->exitmode();
		}
	}
}
class USLException extends Exception{}