<?php
/**
 * @name switch.class.php
 * @description Classe Switch du plugin "switch" du projet ESIA. 
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 17/04/2013
 */

include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once LIB_SNMP;
include_once TELNET_CLASS;
include_once SSH_CLASS;
//inteface qui doit être implémentée par la classe switch
interface switchInterface
{
	public function setAllInterfaceBySNMP();
}

class Switchs extends Device implements switchInterface
{
	const VERSION_1 = 1;
	const VERSION_2C= 2;
	const VERSION_3 = 3;
	protected $interface_class;
	protected $list_interface;
	protected $template_html;
	protected $telnetclass;
	protected $sshclass;
	protected $template_conf;
	protected $oidManufacturer;
	/*@NAME: __construct
	 *@IN: /
	*@OUT: /
	*@TREAT: constructeur de base 
	*@EXCEPTION: /
	*/
	public function __construct($dev=null)
	{
		if(@get_class($dev)=='Device')
		{
			parent::constructDevice($dev);
		}
		else
		{
			parent::__construct();
		}
		$this->telnetclass='Telnet';
		$this->sshclass='ssh';
		$this->list_interface=array();
		$this->interface_class='interfaces';
	}
	/*@NAME: getInstanceNetworkInterface
	 *@IN: $i le ifindex de l'interface
	*@OUT:  /
	*@TREAT: Instance dynamiquement un objet "Interfaces" ou un de ses enfants
	*@EXCEPTION: /
	*/
	public function getInstanceNetworkInterface($i=null)
	{
		if(!empty($i)&&is_numeric($i))
		{
			$this->interfaces[$i]=new  $this->interface_class($this->snmpReq,$this->ip);
			return $this->interfaces[$i];
		}
		else
		{
			$a = new  $this->interface_class($this->snmpReq,$this->ip);
			$this->interfaces[]=$a;
			return $a;
		}
	}
	/*@NAME: getConfigurationTemplate
	 *@IN: /$tpl
	*@OUT: le chemin d'accès au fichier de template de configuration
	*@TREAT: retourne le chemin d'accès au fichier de template de configuration
	*@EXCEPTION: /
	*/
	public function getConfigurationTemplate()
	{
		return $this->template_conf;
	}
	/*@NAME: getOIDManufacturer
	 *@IN:  /
	*@OUT:  retourne un numérique équivalent à l'oid du fabricant
	*@TREAT: retourne le chiffre que l'IANA à donné au fabricant
	*@EXCEPTION: /
	*/
	public function getOIDManufacturer()
	{
		return $this->oidManufacturer;
	}
	/*@NAME: setConfigurationTemplate
	 *@IN: $tpl le chemin d'accès au fichier de template de configuration
	*@OUT:  /
	*@TREAT: set le chemin d'accès au fichier de template de configuration
	*@EXCEPTION: /
	*/
	public function setConfigurationTemplate($tpl)
	{
		$this->template_conf=$tpl;
	}
	/*@NAME: setHTMLTemplate
	 *@IN: $tpl le chemin d'accès au fichier de template HTML
	*@OUT:  /
	*@TREAT: set le chemin d'accès au fichier de template HTML
	*@EXCEPTION: /
	*/
	public function setHTMLTemplate($tpl)
	{
		$this->template_html=$tpl;
	}
	/*@NAME: setManufacturerBySNMP
	 *@IN:  /
	*@OUT:  retourne un numérique équivalent à l'oid du fabricant
	*@TREAT: retourne le chiffre que l'IANA à donné au fabricant et set le fabricant au passage
	*@EXCEPTION: DeviceSNMPException si une erreur survient
	*/
	public function setManufacturerBySNMP()
	{
		$this->manufacturer =  $this->getOIDManufacturerBySNMP();
		return $this->manufacturer;
	}
	/*@NAME: setModelBySNMP
	 *@IN: /
	*@OUT:  le modèle du switch donné par SNMP 
	*@TREAT: set le modèle du switch DOIT être redéfinie dans les classe fille
	*@EXCEPTION:  DeviceSNMPException si une erreur survient
	*/	
	public function setModelBySNMP()
	{
		return false;
	}
	/*@NAME: setOSBySNMP
	 *@IN: /
	*@OUT:  l'OS du switch donné par SNMP
	*@TREAT: set l'OS du switch DOIT être redéfinie dans les classe fille
	*@EXCEPTION:  DeviceSNMPException si une erreur survient
	*/
	public function setOSBySNMP()
	{
		return false;
	}
	/*@NAME: setOSVersionBySNMP
	 *@IN: /
	*@OUT:  la version de l'OS du switch donné par SNMP
	*@TREAT: set la version de l'OS du switch DOIT être redéfinie dans les classe fille
	*@EXCEPTION:  DeviceSNMPException si une erreur survient
	*/
	public function setOSVersionBySNMP()
	{
		return false;
	}
	/*@NAME: getRMONAlarmVariable
	 *@IN: /
	*@OUT:  la liste des alarmes et le ifIndex exemple: IF-MIB::ifInErrors.10032
	*@TREAT: récupère les variable alarm de rmon SNMP NEED:formatRmonAlarm (SNMPResponse)
	*@EXCEPTION:  DeviceSNMPException si une erreur survient 
	*/
	public function getRMONAlarmVariable()
	{
		$oid =".1.3.6.1.2.1.16.3.1.1.3";
		$a = parent::snmpwalk($oid);
		$liste= array();
		foreach ($a as $alarm)
		{
			$liste[]=$alarm;
		}
		return $liste;
	}
	/*@NAME: setAllInterfaceBySNMP
	 *@IN: /
	*@OUT: /
	*@TREAT: récupère en SNMP la liste de toutes les interfaces et instancie les objets correspondant
	*@EXCEPTION:  DeviceSNMPException si une erreur survient
	*/
	public function setAllInterfaceBySNMP()
	{
		$oid = ".1.3.6.1.2.1.2.2.1.3";
		$inter = parent::snmpwalkoid($oid);
		for(reset($inter); $i = key($inter); next($inter))
		{
			try
			{
				if (preg_match("#ethernetCsmacd#",$inter[$i])|| preg_match("#6#",$inter[$i])||preg_match("#gigabitEthernet#",$inter[$i])|| preg_match("#117#",$inter[$i])|| preg_match("#131#",$inter[$i]) || preg_match("#135#",$inter[$i]))
				{
					if (preg_match("#([0-9]*$)#",$i,$regs))
					{
						$oid = ".1.3.6.1.2.1.31.1.1.1.1.".$regs[1];
						try
						{
							$name = parent::snmpget($oid);
						}
						catch(DeviceSNMPException $e)
						{
							$oid = ".1.3.6.1.2.1.2.2.1.2.".$regs[1];
							$name = parent::snmpget($oid);
						}
						$name = formatString($name);
						$name=preg_replace ('#^ #','',$name);
						$name=preg_replace ('# $#','',$name);
						if($name=='')
						{
							$oid = ".1.3.6.1.2.1.2.2.1.2.".$regs[1];
							$name = formatString(parent::snmpget($oid));
						}
						if(preg_match("#VL#",$name)|| preg_match("#Nu#",$name))
						{
						}
						else
						{
							$int = new $this->interface_class($regs[1],
									'ethernetCsmacd',
									$name,
									$this->snmpReq,
									$this->manufacturer,
									$this->ip);
							$this->interfaces[$regs[1]]=$int;
						}
					}
				}
			}
			catch(Exception $e)
			{
				
			}
		}
	}
	public function getInterfacesMib($infos="all")
	{	
		$oid = ".1.3.6.1.2.1.2.2.1.3";
		$inter = parent::snmpwalkoid($oid);
		for(reset($inter); $i = key($inter); next($inter))
		{
//			if (preg_match("#ethernetCsmacd#",$inter[$i])|| preg_match("#6#",$inter[$i])||preg_match("#gigabitEthernet#",$inter[$i])|| preg_match("#117#",$inter[$i])|| preg_match("#131#",$inter[$i]) || preg_match("#135#",$inter[$i])|| preg_match("#161#",$inter[$i]))
//			{
				if (preg_match("#([0-9]*$)#",$i,$regs))
				{
					if( $infos != "index"){
						$oid = ".1.3.6.1.2.1.31.1.1.1.1.".$regs[1];
						try
						{
							$name = parent::snmpget($oid);
						}
						catch(DeviceSNMPException $e)
						{
							try
							{
								$oid = ".1.3.6.1.2.1.2.2.1.2.".$regs[1];
								$name = parent::snmpget($oid);
							}
							catch(Exception $e)
							{
							}
						}
						$name = formatString($name);
						$name=preg_replace ('#^ #','',$name);
						$name=preg_replace ('# $#','',$name);
						if($name=='')
						{
							$oid = ".1.3.6.1.2.1.2.2.1.2.".$regs[1];
							$name = formatString(parent::snmpget($oid));
						}
					}
					$int = new $this->interface_class($regs[1],
								'ethernetCsmacd',
								$name,
								$this->snmpReq,
								$this->manufacturer,
								$this->ip);
					try
					{
						$int->ifIndex = $regs[1];
						if( $infos != "index" ){
							$int->getDescriptionBySNMP();
							$int->getOperStatusBySNMP();	
						}
						if( $infos == "all" || $infos == "short"){
							$int->getTypeBySNMP();	
							$int->getSpeedSNMP();
						}
						if( $infos == "all" ){
							$int->getIfAliasBySNMP();
							$int->getAdminStatusBySNMP();
							$int->getIfNameBySNMP();
							$int->getMTUBySNMP();
						}
					}
					catch(Exception $e)
					{
					}
					$this->interfaces[$int->ifIndex]=$int;
				}
			}
//		}
		return $this->interfaces;
	}
	public function getInterfaceStat($ifindex,$infos="all")
	{
		try 
		{
		//echo $this->snmpReq->getVersion().' '.$this->snmpReq->getCommunity();
		$int = new $this->interface_class($ifindex,'ethernetCsmacd',
								'',
								$this->snmpReq,
								$this->manufacturer,
								$this->ip);
					$int->ifIndex = $ifindex;
					if( $infos == 'index' ) return $int;
					
					$int->getInErrorsBySNMP();
					$int->getOutErrorsBySNMP();
					$int->getOutOctetsBySNMP();
					$int->getInOctetsBySNMP();
					if( $infos == 'brief' ) return $int;
					
					$int->getIfAliasBySNMP();
					$int->getAdminStatusBySNMP();
					$int->getMTUBySNMP();
					$int->getSpeedSNMP();
					if( $infos == 'short' ) return $int;
					
					$int->getDescriptionBySNMP();
					$int->getTypeBySNMP();
					$int->getOperStatusBySNMP();
					$int->getIfNameBySNMP();
		}
		catch(Exception $e)
		{
			
		}
			return $int;	
	}
	/*@NAME: setAllVlanInterfacebySNMP
	 *@IN: /
	*@OUT: /
	*@TREAT: recupère le Vlan de toute les interfaces du switch DOIT ÊTRE REDEFINIE dans la classe fille
	*@EXCEPTION:  SwitchUnsupportedManufacturerException si une erreur survient
	*/
	public function setAllVlanInterfacebySNMP()
	{
		//throw new SwitchUnsupportedManufacturerException('switch unsuported');
	}
	/*@NAME: setAllModeInterfacebySNMP
	 *@IN: /
	*@OUT: /
	*@TREAT: recupère le mode de toute les interfaces du switch DOIT ÊTRE REDEFINIE dans la classe fille
	*@EXCEPTION:  SwitchUnsupportedManufacturerException si une erreur survient
	*/
	public function setAllModeInterfacebySNMP()
	{
		//throw new SwitchUnsupportedManufacturerException('switch unsuported');
	}
	/*@NAME: getModeDataBase
	 *@IN: /
	*@OUT: /
	*@TREAT: recupère le mode de fonctionnenment des interfaces du switch DOIT ÊTRE REDEFINIE dans la classe fille
	*@EXCEPTION:  SwitchUnsupportedManufacturerException si une erreur survient
	*/
	public function getModeDataBase()
	{
		
	}
	/*@NAME: getAllInterface
	 *@IN: /
	*@OUT:retourne un tableau d'objet 'interface' ou une de ses filles
	*@TREAT: retourne un tableau d'objet 'interface' ou une de ses filles
	*@EXCEPTION: /
	*/
	public function getAllInterface()
	{
		return $this->interfaces;
	}
	/*@NAME: getInterface
	 *@IN: $i le ifindex de l'interface
	*@OUT:retourne un objet 'interface' ou une de ses filles
	*@TREAT: retourne un objet 'interface' ou une de ses filles
	*@EXCEPTION: /
	*/
	public function getInterface($i)
	{
		return $this->interfaces[$i];
	}
	/*@NAME: getCPUusage
	 *@IN: /
	*@OUT: l'usage du CPU
	*@TREAT: recupère l'utilisation CPU du switch DOIT ÊTRE REDEFINIE dans la classe fille
	*@EXCEPTION:  DeviceException si une erreur survient
	*/	
	public function getCPUusage()
	{
		return false;
	}
	/*@NAME: getTemperature
	 *@IN: /
	*@OUT: la température du switch
	*@TREAT: recupère la température du switch DOIT ÊTRE REDEFINIE dans la classe fille
	*@EXCEPTION:  DeviceException si une erreur survient
	*/
	public function getTemperature()
	{
		return false;
	}
	/*@NAME: getVlanDataBase
	 *@IN: /
	*@OUT: un tableau contenant la VLAN DATABASE
	*@TREAT: recupère la VLAN DATABASE du switch DOIT ÊTRE REDEFINIE dans la classe fille
	*@EXCEPTION:  DeviceException si une erreur survient
	*/
	public function getVlanDataBase()
	{
		return false;
	}
	/*@NAME: countInterface
	 *@IN: /
	*@OUT: retourne le nombre d'interface du switch
	*@TREAT: retourne le nombre d'interface du switch
	*@EXCEPTION: /
	*/
	public function countInterface()
	{
		return count($this->list_interface);
	}
	/*@NAME: getHTMLTemplate
	 *@IN: /
	*@OUT:  le chemin d'accès au template HTML
	*@TREAT: retourne le chemin d'accès au fichier de template HTML
	*@EXCEPTION: /
	*/
	public function getHTMLTemplate()
	{
		return $this->template_html;
	}

	/*@NAME: command
	 *@IN: $com la commande à uploadé
	*@OUT: La sortie de la commande demandée
	*@TREAT: l'upload la commande demandée sur le switch 
	*@EXCEPTION: SwitchConnectionFailedException si une erreur survient
	*			SwitchCommandNoPresentInTemplateExceptionsi la command en'est pas dans le template
	*/
	public function command($com)
	{
		$tab=array();
		if(!empty($com))
		{
				try
				{
					if($this->typeConnection=='TELNET')
					{
						$t = new $this->telnetclass($this->ip);
					}
					else
					{
						$t = new $this->sshclass($this->ip);
					}
					$t->login($this->login,$this->pwd);
					$t->setTimer(200000);
					$output = $t->cmd($this->conf->getConfiguration());
					$output = preg_replace('#.*\[[0-9]*;[0-9]*H#','',$output);
					$tab = explode("\n",$output);
					$t->close();
				}
				catch(TelnetException $e)
				{
					throw new SwitchConnectionFailedException($e->getMessage());
				}
				catch(SSHException $e)
				{
					throw new SwitchConnectionFailedException($e->getMessage());
				}
		}
		else
		{
			throw new SwitchException('command is empty');
		}
		return $tab;
	}

	/*@NAME: uploadFileConfiguration
	 *@IN: $filename chemin d'accès au fichier de configuration 
	*@OUT: /
	*@TREAT: charge un fichier de configuration sur le switch 
	*@EXCEPTION: SwitchException si une erreur survient 
	*/
	public function uploadFileConfiguration($filename)
	{
		$file=@fopen($filename,"r");
		//si on est pas arriver à ouvrir le fichier
		if($file===FALSE)
		{
			throw new SwitchException('File no found: '.$filename);
		}
		try 
		{
			if($this->typeConnection=='TELNET')
			{
				$connection = new $this->telnetclass($this->ip);
			}
			else
			{
				$connection = new $this->sshclass($this->ip);
			}
			$connection->login($this->login, $this->pwd);
			$output="";
			while (!feof($file))
			{
				/*On lit la ligne courante*/
				$buffer = fgets($file);
				$buffer = rtrim($buffer);
				/*on l'upload*/
				$output .= $connection->cmd($buffer);
			}
			$connection->close();
		}
		catch(TelnetException $e)
		{
			throw new SwitchConnectionFailedException($e->getMessage());
		}
		catch(SSHException $e)
		{
			throw new SwitchConnectionFailedException($e->getMessage());
		}
		fclose($file);
		return $output;
	}
	/*@NAME: uploadConfiguration
	 *@IN: $conf une chaine de caractèree contenant la conf à uploader
	*@OUT: /
	*@TREAT: charge une configuration sur le switch
	*@EXCEPTION: SwitchConnectionFailedException si une erreur survient
	*/
	public function uploadConfiguration($conf)
	{
		try
		{
			if($this->typeConnection=='TELNET')
			{
				$connection = new $this->telnetclass($this->ip);
			}
			else
			{
				$connection = new $this->sshclass($this->ip);
			}
			$connection->login($this->login, $this->pwd);
			$arr = explode("\n",$conf);
			$output="";
			foreach ($arr as $line)
			{
				$out =$connection->cmd($line);
				$output.=$out;
			}
			$connection->close();
		}
		catch(TelnetException $e)
		{
			throw new SwitchConnectionFailedException($e->getMessage());
		}
		catch(SSHException $e)
		{
		    echo $e->getMessage();
			throw new SwitchConnectionFailedException($e->getMessage());
		}
		return $output;
	}
	public function getManufacturerCSS()
	{
		return "";
	}
}
class SwitchException extends Exception{}
class SwitchNotAliveSNMPException extends SwitchException{}
class SwitchNotAlivePingException extends SwitchException{}
class SwitchConnectionFailedException extends SwitchException{}
class SwitchUnsupportedManufacturerException extends SwitchException{}
class SwitchNoConfigurationTemplateException extends SwitchException{}
class SwitchCommandNoPresentInTemplateException extends SwitchException{}
class SwitchSNMPException extends SwitchException{}
?>
