<?php
/**
 * @name  Model_switch
* @description Classe héritant de "Model_esia" gèrent tout les accès à la base de donnée du plugin Switch
* @author Nicolas Biersart <n.biersart@esia-network.com>
* @contributor
* @copyright Nicolas Biersart 2012
* @license
* @esia 1.00
* @version 1.0.0
* @date 08/11/2012
*/
include_once SNMP_CLASS;
include_once DEVICE_CLASS;
include_once MVCMODEL;
$arr = Model_esia::getWebPlugin($dbconnection,false,'Switch');
$sw_path= $path.$arr[0]['apache_path'];
include_once $sw_path.'/libs/RemoteSwitch.class.php';
include_once $sw_path.'/libs/switch.class.php';

class Model_switch extends Model_esia
{
	/*@NAME: InstanceSwitch
	 *@IN: $dbconnection un objet PDO
	* 	$switch nom ou ID du serveur
	*	$manufacturer (optionnelle) pas encore implémenté
	*@OUT: un objet Switch
	*@TREAT: Instance un objet switch
	*@EXCEPTION: 	EsiaDbException  si une erreur avec la BD survient
	*/
	public static function InstanceSwitch($dbconnection,$switch,$manufacturer=null)
	{
		try
		{
			$node = Model_esia::InstanceNode($dbconnection,$switch);
			$sw = New Switchs($node);
			if(@get_class($node)=='RemoteDevice')
			{
				$sw = New RemoteSwitch($node,$node->getGESA() );
			}
			else
			{	
				//lancement de l'USL
				$query= $dbconnection->prepare("SELECT model_name,model_html_tpl,os_name,os_version,os_template_path, manufacturer_name,manufacturer_logo,class_path,class_name
						FROM esia_switch
						LEFT JOIN esia_php_class ON switch_class_id = class_id
						LEFT JOIN esia_os ON switch_os_id = os_id
						LEFT JOIN esia_model ON switch_model_id = model_id
						LEFT JOIN esia_manufacturer ON model_manufacturer_oid = manufacturer_oid
						WHERE switch_device_id = ?;");
				$query = parent::executeStatement($query,array($node->getId()));
				if($row=$query->fetch(PDO::FETCH_OBJ))
				{
					//on regarde si il n'y a pas une classe de gestion pour le switch
					if(isset($row->class_path)&&!empty($row->class_path)&& isset($row->class_name)&&!empty($row->class_name) )
					{
						include_once $row->class_path;
						$sw = new $row->class_name;
					}
					$sw->constructDevice($node);
					$sw->setManufacturer($row->manufacturer_name);
					$sw->setOSVersion($row->os_version);
					$sw->setOS($row->os_name);
					$sw->setModel($row->model_name);
					if(!empty($row->manufacturer_logo))
					{
						$sw->setManufacturerLogo($row->manufacturer_logo);
					}
					if(!empty($row->model_html_tpl))
					{
						$sw->setHTMLTemplate($row->model_html_tpl);
					}
					if(!empty($row->os_template_path))
					{
						$sw->setConfigurationTemplate($row->os_template_path);
					}
				}
				//on récupère toute les interfaces du switch
			/*	$query=$dbconnection->prepare("SELECT interface_id,interface_name,interface_ifindex,interface_plug,interface_desc,interface_lastchange_username,to_char(interface_lastchange_time, 'DD/MM/YYYY') AS interface_lastchange_time ,interface_down_time,interface_critical
						FROM esia_interface
						WHERE interface_device_id =?
						ORDER BY interface_ifindex;");
				$query= parent::executeStatement($query,array($sw->getId()));
				while($r=$query->fetch(PDO::FETCH_OBJ))
				{
					$sw->getInstanceNetworkInterface($r->interface_ifindex);
					$sw->getInterface($r->interface_ifindex)->Id = $r->interface_id;
					$sw->getInterface($r->interface_ifindex)->ifName = $r->interface_name;
					$sw->getInterface($r->interface_ifindex)->ifIndex = $r->interface_ifindex;
					$sw->getInterface($r->interface_ifindex)->description = $r->interface_desc;
					$sw->getInterface($r->interface_ifindex)->plug = $r->interface_plug;
					$sw->getInterface($r->interface_ifindex)->critical  = $r->interface_critical;
					$sw->getInterface($r->interface_ifindex)->timeLastChange = $r->interface_lastchange_time;
					$sw->getInterface($r->interface_ifindex)->userLastChange = $r->interface_lastchange_username;
					$sw->getInterface($r->interface_ifindex)->ifLastChange = $r->interface_down_time;
				}*/
			}
			return $sw;
		}
		catch(EsiaDbException $e)
		{
			throw new ModelSwitchException('Errors in instanciation Switch\'s object'.$switch,1,$e);
		}
	}
	/*@NAME getInstanceManufacturerSwitch
	 *@IN: $dbconnection un objet PDO
	*	$manufacturer oid du fabricant
	*@OUT: un objet switch ou une classe fille
	*@TREAT: instancie un objet switch ou la classe fille de "switchs" propre au fabricant
	*@EXCEPTION: EsiaDbException si une erreur survient
	*/
	public static function getInstanceManufacturerSwitch($dbconnection,$manufacturer)
	{
		$query= $dbconnection->prepare("SELECT class_path,class_name
				FROM esia_php_class
				WHERE class_manufacturer_oid = ? AND class_type='switch';");
		$query = parent::executeStatement($query,array($manufacturer));
		$row=$query->fetch(PDO::FETCH_OBJ);
		//on regarde si il n'y a pas une classe de gestion pour le switch
		if(isset($row->class_path)&&!empty($row->class_path)&& isset($row->class_name)&&!empty($row->class_name) )
		{
		//	echo $row->class_path.'<br />';
			include_once $row->class_path;
			$sw = new $row->class_name;
		}
		else
		{
			$sw = new Switchs();
		}
		return $sw;
	}
	/*@NAME getSwitchClass
	 *@IN: $dbconnection un objet PDO
	*	$x id du switch (si on recherche via le fabricant mettre se paramètre à NULL)
	*	$manufacturer  (optionnelle) oid du fabricant
	*@OUT: un objet contenant comme variable le class_name et le class_path
	*@TREAT: retourne les infos de la classe du fabricant gérant les switch
	*@EXCEPTION: EsiaDbException si une erreur survient
	*/
	static function getSwitchClass($dbconnection,$x ,$manufacturer=NULL)
	{
		$id=$x;
		if(!empty($manufacturer)&&is_numeric($manufacturer))
		{
			$query= $dbconnection->prepare("SELECT class_name,class_path
					FROM esia_manufacturer
					JOIN esia_manufacturer_class ON manufacturer_oid = class_manufacturer_oid
					WHERE class_type='switch' AND manufacturer_oid=? ;");
			$id= $manufacturer;
		}
		else
		{
			if(is_numeric($x))
			{
				$query= $dbconnection->prepare("SELECT class_name,class_path
						FROM esia_manufacturer_class
						JOIN esia_manufacturer ON manufacturer_oid = class_manufacturer_oid
						JOIN esia_switch ON switch_manufacturer_oid = manufacturer_oid
						WHERE switch_device_id=? ;");
			}
			else
			{
				$query= $dbconnection->prepare("SELECT class_name,class_path
						FROM esia_manufacturer_class
						JOIN esia_manufacturer ON manufacturer_oid = class_manufacturer_oid
						JOIN esia_switch ON switch_manufacturer_oid = manufacturer_oid
						JOIN esia_device ON switch_device_id = device_id
						WHERE device_type='switch' AND device_name=? ;");
			}
		}
		$query= parent::executeStatement($query,array($id));
		if($row=$query->fetch(PDO::FETCH_OBJ))
		{
			return $row;
		}
		else
		{
			return null;
		}
	}
	/*@NAME getWebSwitchTemplate
	 *@IN: $dbconnection un objet PDO
	*	$model le modèle du switch
	*@OUT: le chemin d'accès au template HTML du switch
	*@TREAT: retourne le chemin d'accès au template HTML du switch
	*@EXCEPTION: EsiaDbException si une erreur survient
	*/
	public function getWebSwitchTemplate($dbconnection,$model)
	{
		$query = $dbconnection->prepare("SELECT model_html_tpl FROM esia_model WHERE model_name =?;");
		$query= parent::executeStatement($query,array($model));
		$r_tpl=$query->fetch(PDO::FETCH_OBJ);
		return $r_tpl->model_html_tpl;
	}
	/*@NAME getWebSwitchTemplate
	 *@IN: $dbconnection un objet PDO
	*@OUT: une requète préparé PDO prenant 1 paramètre l'id de l'interface
	*@TREAT: prépare une requète pour récupérer toutes les erreurs d'une interface donnée
	*@EXCEPTION: /
	*/
	public function prepareGetInterfaceErrors($dbconnection)
	{
		$q_error=$dbconnection->prepare("
				SELECT error_type,error_name,happen_message,happen_value
				FROM esia_interface
				JOIN esia_error_happen ON happen_interface_id=interface_id
				JOIN esia_errors ON error_id = happen_error_id
				WHERE  happen_acquitte <>1 AND interface_id =?
				ORDER BY happen_timestamp DESC;");
		return $q_error;
	}
	/*@NAME getInterfaceErrors
	 *@IN: $dbconnection un objet PDO ou un PDOStatement
	*	$id identifiant de l'interface
	*@OUT: un tableau associatif contenant
	*	$error[]['type']	type de l'erreur (critical,warning,etc)
	*	$error[]['name']	nom de l'erreur
	*	$error[]['message']	message
	*	$error[]['value']	valeur de l'erreur
	*@TREAT: si il s'agit d'un objet PDO on préparera la requête
	*		si c'est un PDOStatement (on a préparé la requete par la méthode model_sw::prepareGetInterfaceErrors()
			*		on exécutera la requète
			*		cette méthode retourne toute les erreurs actives de l'interfaces
			*@EXCEPTION: EsiaDbException si une erreur survient
			*/
	public function getInterfaceErrors($dbconnection,$id)
	{
		$query='';
		if(get_class($dbconnection)=='PDO')
		{
			$query=$dbconnection->prepare("
					SELECT error_type,error_name,happen_message,happen_value
					FROM esia_interface
					JOIN esia_error_happen ON happen_interface_id=interface_id
					JOIN esia_errors ON error_id = happen_error_id
					WHERE  happen_acquitte <>1 AND interface_id =?
					ORDER BY happen_timestamp DESC;");
		}
		elseif(get_class($dbconnection)=='PDOStatement')
		{
			$query = $dbconnection;
		}
		else
		{
			throw new EsiaDbException('$dbconnection is not a PDO / PDOStatement object');
		}
		$query= parent::executeStatement($query,array($id));
		$error=array();
		while($r_error=$query->fetch(PDO::FETCH_OBJ))
		{
			$error[]= array('type'=> $r_error->error_type,
					'name'=>$r_error->error_name,
					'message'=> $r_error->happen_message,
					'value'=> $r_error->happen_value);
		}
		return $error;
	}
	/*@NAME getInterfaceAllError
	 *@IN: $dbconnection un objet PDO ou un PDOStatement
	*	$id identifiant de l'interface
	*@OUT: un tableau associatif contenant
	*	$error[]['type']	type de l'erreur (critical,warning,etc)
	*	$error[]['name']	nom de l'erreur
	*	$error[]['message']	message
	*	$error[]['value']	valeur de l'erreur
	*	$error[]['ack']		si =0 l'erreur est active si =1 erreur acquitté
	*	$error[]['time']	l'heure à l'acquelle l'erreur c'est produite
	*	$error[]['time_ack'] l'heure à l'acquelle l'erreur a été acquitée
	*@TREAT: Retourne toute les erreurs actives et passé d'une interface donnée
	*@EXCEPTION: EsiaDbException si une erreur survient
	*/
	public function getInterfaceAllError($dbconnection,$id)
	{
		$query=$dbconnection->prepare("
				SELECT happen_id,error_name,error_type,happen_message,to_char(happen_timestamp, 'DD/MM/YYYY HH24:MI:SS') as times,happen_acquitte,happen_value,to_char(happen_time_acquitte, 'DD/MM/YYYY HH24:MI:SS') as time_ack
				FROM esia_error_happen
				JOIN esia_errors ON error_id =happen_error_id
				WHERE happen_interface_id=?
				ORDER BY happen_acquitte,happen_timestamp DESC;");
	
		$query= parent::executeStatement($query,array($id));
		$error=array();
		while($r_error=$query->fetch(PDO::FETCH_OBJ))
		{
			$error[]= array('id'=>$r_error->happen_id,
					'type'=> $r_error->error_type,
					'name'=>$r_error->error_name,
					'message'=> $r_error->happen_message,
					'value'=> $r_error->happen_value,
					'ack'=>$r_error->happen_acquitte,
					'time_ack'=>$r_error->time_ack,
					'time'=>$r_error->times);
		}
		return $error;
	}
	/*@NAME getInterfaceListMacAddress
	 *@IN: $dbconnection un objet PDO ou un PDOStatement
	*	$id identifiant de l'interface
	*@OUT: un tableau associatif contenant
	* 		$list_mac[]['mac']	l'adresse mac
	* 		$list_mac[]['vlan']	le vlan utilisé
	* 		$list_mac[]['ip']	l'adresse ip utilisée
	* 		$list_mac[]['time_connection']	date de connection
	* 		$list_mac[]['time_out']	date de  d
	* 		$list_mac[]['statut']
	*@TREAT: Retourne toute les erreurs actives et passé d'une interface donnée
	*@EXCEPTION: EsiaDbException si une erreur survient
	*/
	public function getInterfaceListMacAddress($dbconnection,$id)
	{
	
		$q_mac= $dbconnection->prepare("SELECT mac_address,mac_vlan_id,mac_time_connection,mac_time_out_connection,mac_ip_address
				FROM esia_mac_address
				WHERE mac_interface_id=?
				ORDER BY mac_time_out_connection;");
		$q_mac = parent::executeStatement($q_mac,array($id));
		$list_mac =array();
		while($r_mac=$q_mac->fetch(PDO::FETCH_OBJ))
		{
	
			preg_match("#^([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$#",$r_mac->mac_address,$regs);
			$mac = $regs[1].":".$regs[2].":".$regs[3].":".$regs[4].":".$regs[5].":".$regs[6];
			$mac=strtoupper($mac);
			$status=0;
			if($r_mac->mac_time_out_connection==0)
				$status=1;
			else
				$status=0;
			$list_mac[]=array('mac'=>$mac,
					'vlan'=>$r_mac->mac_vlan_id,
					'ip'=>$r_mac->mac_ip_address,
					'time_connection'=>date("d/m/Y H:i:s",$r_mac->mac_time_connection),
					'time_out'=>date("d/m/Y H:i:s",$r_mac->mac_time_out_connection),
					'statut'=>$status );
		}
		return 	$list_mac;
	}
	/*@NAME: getLogFilePath
	 *@IN: $dbconnection un objet PDO
	*@OUT: le chemin d'accès
	*@TREAT: retourne le chemin d'accès au repertoire de log des switchs
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public static function getLogFilePath($dbconnection)
	{
		$query= $dbconnection->prepare("SELECT config_value, config_description, config_type, config_visibility
				FROM esia_configuration
				WHERE config_key=?;");
		$query = parent::executeStatement($query,array('SW_PATH_LOGFILE_CONFIGURATION'));
		$row = $query->fetch(PDO::FETCH_OBJ);
		return $row->config_value;
	}
	/*@NAME: getSwitchManufacturerClass
	 *@IN: $dbconnection un objet PDO
	*		$oid (optionnel) oid du fabricant
	*@OUT: 	$arr[]['id'] id de la classe
	*		$arr[]['name'] nom de la classe
	*		$arr[]['type'] type de la classe
	*		$arr[]['path'] chemin d'accès à la classe
	*		$arr[]['manufacturer_oid'] oid du fabricant
	*		$arr[]['manufacturer_name'] nom du fabricant
	*@TREAT: retourne les information des classes de gestion
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public static function getSwitchManufacturerClass($dbconnection,$oid=null )
	{
		if(isset($oid))
		{
			$query =  $dbconnection->prepare("SELECT class_id,class_name,class_type,class_path,class_manufacturer_oid,manufacturer_name
					FROM esia_php_class
					JOIN esia_manufacturer ON class_manufacturer_oid = manufacturer_oid
					WHERE class_manufacturer_oid=? AND class_type='switch';");
			$query = parent::executeStatement($query,array($oid));
		}
		else
		{
			$query =  $dbconnection->prepare("SELECT class_id,class_name,class_type,class_path,class_manufacturer_oid,manufacturer_name
					FROM esia_php_class
					JOIN esia_manufacturer ON class_manufacturer_oid = manufacturer_oid
					WHERE class_type='switch';");
			$query = parent::executeStatement($query);
		}
		$arr = array();
		if($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id'=>$row->class_id,
					'name'=> $row->class_name,
					'type'=> $row->class_type,
					'path'=> $row->class_path,
					'manufacturer_oid' => $row->class_manufacturer_oid,
					'manufacturer_name' =>$row->manufacturer_name );
		}
		return $arr;
	}
	/*@NAME: getSwitchOs
	 *@IN: $dbconnection un objet PDO
	*		$version(optionnel) version de l'os
	*		$oid (optionnel) oid du fabricant si $version est à null retourne tout les OS du fabricant
	*@OUT: 	$arr[]['id'] id de l'OS
	*		$arr[]['name'] nom de l'OS
	*		$arr[]['version'] version
	*		$arr[]['path'] chemin d'accès au template de configuration
	*		$arr[]['manufacturer_oid'] oid du fabricant
	*		$arr[]['manufacturer_name'] nom du fabricant
	*@TREAT: retourne les information de sur les OS d'un fabricant
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public static function getSwitchOs($dbconnection,$version=null,$oid=null )
	{
		if(isset($version))
		{
			$query =  $dbconnection->prepare("SELECT os_id, os_name, os_version, os_template_path, os_manufacturer_oid,manufacturer_name
					FROM esia_os
					JOIN esia_manufacturer ON os_manufacturer_oid = manufacturer_oid
					WHERE os_version = ?;");
			$query = parent::executeStatement($query,array($version));
		}
		elseif(isset($oid))
		{
			$query =  $dbconnection->prepare("SELECT os_id, os_name, os_version, os_template_path, os_manufacturer_oid,manufacturer_name
					FROM esia_os
					JOIN esia_manufacturer ON os_manufacturer_oid = manufacturer_oid
					WHERE os_manufacturer_oid = ?;");
			$query = parent::executeStatement($query,array($oid));
		}
		else
		{
			$query =  $dbconnection->prepare("SELECT os_id, os_name, os_version, os_template_path, os_manufacturer_oid,manufacturer_name
					FROM esia_os
					JOIN esia_manufacturer ON os_manufacturer_oid = manufacturer_oid;");
			$query = parent::executeStatement($query);
		}
		$arr = array();
		if($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id'=>$row->os_id,
					'name'=> $row->os_name,
					'version'=> $row->os_version,
					'path'=> $row->os_template_path,
					'manufacturer_oid' => $row->os_manufacturer_oid,
					'manufacturer_name' =>$row->manufacturer_name );
		}
		return $arr;
	}
	/*@NAME: getSwitchModel
	 *@IN: $dbconnection un objet PDO
	*		$model(optionnel) nom du modèle
	*		$oid (optionnel) oid du fabricant si $version est à null retourne tout les modèle du fabricant
	*@OUT: 	$arr[]['id'] id
	*		$arr[]['name'] nom du modèle
	*		$arr[]['html_tpl'] chemin d'accès au template HTML
	*		$arr[]['manufacturer_oid'] oid du fabricant
	*		$arr[]['manufacturer_name'] nom du fabricant
	*@TREAT: retourne les information de sur les OS d'un fabricant
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public static function getSwitchModel($dbconnection,$model=null,$oid=null )
	{
		if(isset($model))
		{
			$query =  $dbconnection->prepare("SELECT model_id, model_name, model_manufacturer_oid, model_html_tpl,manufacturer_name
					FROM esia_model
					JOIN esia_manufacturer ON model_manufacturer_oid = manufacturer_oid
					WHERE model_name = ?;");
			$query = parent::executeStatement($query,array($model));
		}
		elseif(isset($oid))
		{
			$query =  $dbconnection->prepare("SELECT model_id, model_name, model_manufacturer_oid, model_html_tpl,manufacturer_name
					FROM esia_model
					JOIN esia_manufacturer ON model_manufacturer_oid = manufacturer_oid
					WHERE model_manufacturer_oid = ?;");
			$query = parent::executeStatement($query,array($oid));
		}
		else
		{
			$query =  $dbconnection->prepare("SELECT model_id, model_name, model_manufacturer_oid, model_html_tpl,manufacturer_name
					FROM esia_model
					JOIN esia_manufacturer ON model_manufacturer_oid = manufacturer_oid;");
			$query = parent::executeStatement($query);
		}
		$arr = array();
		if($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id'=>$row->model_id,
					'name'=> $row->model_name,
					'html_tpl'=> $row->model_html_tpl,
					'manufacturer_oid' => $row->model_manufacturer_oid,
					'manufacturer_name' =>$row->manufacturer_name );
		}
		return $arr;
	}
	/*@NAME: getInterfaceInformation
	 *@IN: $dbconnection un objet PDO
	*		$nodeid id du switch
	*		$ifindex (optionnel) ifindex de l'interface
	*@OUT: $arr[$r->interface_ifindex]= array(
					'ifindex'=>$r->interface_ifindex,
					'id'=> $r->interface_id,
					'description'=>$r->interface_desc,
					'plug' => $r->interface_plug,
					'critical' => $r->interface_critical,
					'timeLastChange' => $r->interface_lastchange_time,
					'userLastChange' => $r->interface_lastchange_username,
					'ifLastChange' => $r->interface_down_time,
	*@TREAT: retourne les informations en base de donnée sur les interfaces
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function getInterfaceInformation($dbconnection,$nodeid,$ifindex=null)
	{
		$sql = "SELECT interface_id,interface_name,interface_ifindex,interface_plug,interface_desc,interface_lastchange_username,to_char(interface_lastchange_time, 'DD/MM/YYYY') AS interface_lastchange_time ,interface_down_time,interface_critical
		 		FROM esia_interface
				WHERE interface_device_id =? ";
		$qparam=array($nodeid);
		if(isset($ifindex))
		{
			if(is_numeric($ifindex))
			{
				$sql .="AND interface_ifindex=?";
				$qparam[]=$ifindex;
			}
			else
			{
				$sql .="AND interface_name=?";
				$qparam[]=$ifindex;
			}
		}

		$query=$dbconnection->prepare($sql);
		$query= parent::executeStatement($query,$qparam);
		$arr = array();
		while($r=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$r->interface_ifindex]= array(
					'ifindex'=>$r->interface_ifindex,
					'id'=> $r->interface_id,
					'description'=>$r->interface_desc,
					'plug' => $r->interface_plug,
					'critical' => $r->interface_critical,
					'timeLastChange' => $r->interface_lastchange_time,
					'userLastChange' => $r->interface_lastchange_username,
					'ifLastChange' => $r->interface_down_time,
					);
		}
		return $arr;
	}
	/*@NAME: insertSwitch
	 *@IN: $dbconnection un objet PDO
	*		$sw un objet switch
	*@OUT: /
	*@TREAT: Insère un switch dans la base de donnée (qui a été préalable inséré dans la table esia_device (fonction model_esia::addNode(...) )
			*@EXCEPTION: 	EsiaDbException si une erreur survient
			*/
	public function insertSwitch($dbconnection,Switchs $sw)
	{
		//on récupère toute les informations (OS,class de gestion,etc)
		$modelid = $sw->getModel();
		if(!empty($modelid) )
		{
			$modelid = Model_switch::getSwitchModel($dbconnection,$sw->getModel());
			$modelid = $modelid[0];
		}
		$osid = $sw->getOSversion();
		if(!empty($osid) )
		{
			$osid = Model_switch::getSwitchOs($dbconnection,$sw->getOSversion());
			$osid = $osid[0];
		}
		$classid = $sw->getOIDManufacturer();
		if(!empty($classid) )
		{
			$classid = Model_switch::getSwitchManufacturerClass($dbconnection,$sw->getOIDManufacturer());
			$classid = $classid[0];
		}
		$query = $dbconnection->prepare("INSERT INTO esia_switch(switch_device_id, switch_model_id, switch_class_id, switch_os_id)
				VALUES (:id,:model,:class,:os);");
		$query = parent::executeStatement($query,array(':id' => $sw->getId(),
				':model'=>$modelid['id'],
				':os'=>$osid['id'],
				':class'=>$classid['id']));
		$query = $dbconnection->prepare("INSERT INTO esia_interface(interface_id,interface_name,interface_type,interface_device_id,interface_ifindex,interface_down_time)
				VALUES (DEFAULT,:name,:type,:dev_id,:ifindex,:lastOut);");
		foreach($sw->getAllInterface() as $int)
		{
			$result = $query->execute(array(':name'=>$int->ifName,
					':type'=>$int->ifType,
					':dev_id'=>$sw->getId(),
					':ifindex'=>$int->ifIndex,
					':lastOut'=>$int->ifLastChange));
		}
	}
	/*@NAME: insertSwitchInterfaces
	 *@IN: $dbconnection un objet PDO
	*		$sw un objet switch
	*@OUT: /
	*@TREAT: Insère toutes les interfaces du switch dabs la DB
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function insertSwitchInterfaces($dbconnection,$sw)
	{
		$err=false;
		$query = $dbconnection->prepare("INSERT INTO esia_interface(interface_id,interface_name,interface_type,interface_device_id,interface_ifindex,interface_down_time)
				VALUES (DEFAULT,:name,:type,:dev_id,:ifindex,:lastOut);");
		foreach($sw->getAllInterface() as $int)
		{
			$result = $query->execute(array(':name'=>$int->ifName,
					':type'=>$int->ifType,
					':dev_id'=>$sw->getId(),
					':ifindex'=>$int->ifIndex,
					':lastOut'=>$int->lastout));
			if($result==false)
			{
				$t = $query->errorInfo();
				$err.=$t[2].'<br />';
			}
		}
		if($err!==false)
		{
			throw new EsiaDbException($err);
		}
	}
	/*@NAME: updateSwitch
	 *@IN: $dbconnection un objet PDO
	*		$sw un objet switch
	*@OUT: /
	*@TREAT: mets à jour toutes les informations d'un switch
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function updateSwitch($dbconnection, $sw, $int=null)
	{
		//on récupère toute les informations (OS,class de gestion,etc)
		$modelid = $sw->getModel();
	//	echo $modelid;
		if(!empty($modelid) )
		{
			$modelid = Model_switch::getSwitchModel($dbconnection,$sw->getModel());
			$modelid = $modelid[0];
		//	print_r($modelid);
		}
		$osid = $sw->getOSversion();
		if(!empty($osid) )
		{
			$osid = Model_switch::getSwitchOs($dbconnection,$sw->getOSversion());
			$osid = $osid[0];
		}
		$classid = $sw->getManufacturer();
		if(!empty($classid))
		{
			$classid = Model_switch::getSwitchManufacturerClass($dbconnection,$sw->getManufacturer());
			$classid = $classid[0];
		}
		//echo 'erreur';
		$query = $dbconnection->prepare("UPDATE esia_switch
				SET switch_model_id=:model,switch_os_id=:os,switch_class_id=:class_id
				WHERE switch_device_id=:id;");
		$query = parent::executeStatement($query,array(':id' => $sw->getId(),
				':model'=>$modelid['id'],
				':os'=>$osid['id'],
				':class_id'=>$classid['id']));
		if($query->rowCount()==0)
		{
			$query = $dbconnection->prepare("INSERT INTO esia_switch(switch_device_id, switch_model_id, switch_class_id, switch_os_id)VALUES (:id, :model,:class_id,:os);");
			$query = parent::executeStatement($query,array(':id' => $sw->getId(),
					':model'=>$modelid['id'],
					':os'=>$osid['id'],
					':class_id'=>$classid['id']));
		}
		//Update des interfaces
		//echo 'erreur';
		$oldsw = Model_switch::InstanceSwitch($dbconnection,$sw->getId());
		$oldints = Model_switch::getInterfaceInformation($dbconnection, $sw->getId());
		$newints = $sw->getAllInterface();
		//echo'<pre>'.print_r($newints,true).'</pre>';
		//on prépare la requete
		//echo 'erreur';
		$query = $dbconnection->prepare("UPDATE esia_interface SET interface_down_time=:lastOut,interface_name=:name WHERE interface_id=:id;");
		//comparaison des interfaces entre les switchs pour les requêtes UPDATES
		$deloldint = array();
		foreach($oldints as $oldint)
		{
			$ifindex = $oldint['ifindex'];
			//si on retrouve les mêmes ifindex dans le nouveau switch on update les infor de l'interface
			if(array_key_exists($ifindex,$newints))
			{
				parent::executeStatement($query,array(':id'=>$oldint['id'],
						':name'=>$newints[$ifindex]->ifDescr,
						':lastOut'=>$newints[$ifindex]->lastout));
			}
			else
			{
				//les index ne corresponde pas il se peux qu'on aie changer de fabricant de switch ou de model
				$deloldint[]=$oldint['id'];
			}
		}
		//on va comparer à l'envers dans le cas ou le nouveau switch aurait plus d'interface que l'ancien
		//on insère donc les nouvelles interface
		$query = $dbconnection->prepare("INSERT INTO esia_interface(interface_id,interface_name,interface_type,interface_device_id,interface_ifindex,interface_down_time)
				VALUES (DEFAULT,:name,:type,:dev_id,:ifindex,:lastOut);");
		foreach($newints as $int)
		{
			$ifindex = $int->ifIndex;
			if(array_key_exists($ifindex,$oldints)!==true)
			{
				parent::executeStatement($query,array(':name'=>$int->ifDescr,
						':type'=>$int->ifType,
						':dev_id'=>$sw->getId(),
						':ifindex'=>$int->ifIndex,
						':lastOut'=>$int->lastout));
			}
		}
	
		//on supprime les veilles interfaces qui ne sont pas sur le nouveau switch
		foreach($deloldint as $id)
		{
			Model_switch::deleteSwitchInterface($dbconnection,$id);
		}
	
	
	}
	/*@NAME: updateSwitchInterfaces
	 *@IN: $dbconnection un objet PDO
	*		$sw un objet switch
	*@OUT: /
	*@TREAT: mets à jour toutes les informations des interfaces d'un switch
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function updateSwitchInterfaces($dbconnection,$sw)
	{
		$err=false;
		$query_update = $dbconnection->prepare("UPDATE esia_interface SET interface_ifindex=:ifindex,interface_down_time=:lastOut WHERE interface_id=:id;");
		foreach($sw->getAllInterface() as $int)
		{
			$result = parent::executeStatement($query_update,array(':id'=>$int->getId(),':ifindex'=>$int->getIfIndex(),':lastOut'=>$int->getLastOutput()));
		}
	}
	/*@NAME: updateSwitchInterfaceInformation
	 *@IN: $dbconnection l'objet de connexion PDO
	*		$devid id du switch
	*		$inter	un objet interfaces
	*		$user		login de l'utilisateur
	*@OUT: /
	*@TREAT: met à jour les information de l'interface dans la BD
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function updateSwitchInterfaceInformation($dbconnection,$devid,$inter,$user)
	{
		$query= $dbconnection->prepare("UPDATE esia_interface
				SET interface_plug=:plug,interface_desc=:desc,interface_lastchange_username=:login,interface_lastchange_time=now()
				WHERE interface_device_id = :device_id AND interface_name = :interface_name;");
		parent::executeStatement($query, array(':plug' =>$inter->plug,
				':desc'=>$inter->description,
				':login' =>$user,
				':interface_name'=>$inter->name,
				':device_id'=>$devid));
	}
	/*@NAME: updateSwitchInterfaceCritical
	 *@IN: $dbconnection l'objet de connexion PDO
	*		$devid id du switch
	*		$critical	(0 si non critique, 1 si critique)
	*		$ifindex	ifindex de l'interface
	*		$user		login de l'utilisateur
	*@OUT: /
	*@TREAT: Indique dans la BD si l'interface est critique ou non
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function updateSwitchInterfaceCritical($dbconnection,$devid,$critical,$ifindex,$user)
	{
		$query = $dbconnection->prepare("UPDATE esia_interface
				SET interface_critical=:critical,interface_lastchange_username=:login,interface_lastchange_time=now()
				WHERE interface_device_id = :device_id AND interface_ifindex = :ifindex;");
		parent::executeStatement($query, array(':critical' =>$critical,
				':login' =>$user,
				':ifindex'=>$ifindex,
				':device_id'=>$devid));
	}
	/*@NAME: deleteSwitch
	 *@IN: $dbconnection un objet PDO
	*		$id id du switch
	*@OUT: /
	*@TREAT: supprime le switch de l'application
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function deleteSwitch($dbconnection,$id)
	{
		if(is_numeric($id))
		{
			$query_mac = $dbconnection->prepare("DELETE FROM esia_mac_address WHERE mac_interface_id =?;");
			$query_error = $dbconnection->prepare("DELETE FROM esia_error_happen WHERE happen_interface_id =?;");
			$query = $dbconnection->prepare("SELECT interface_id FROM esia_interface WHERE interface_device_id=?;");
			$query = parent::executeStatement($query,array($id));
			while($row=$query->fetch(PDO::FETCH_OBJ))
			{
				parent::executeStatement($query_mac,array($row->interface_id));
				parent::executeStatement($query_error,array($row->interface_id));
			}
			$query = $dbconnection->prepare("DELETE FROM esia_interface WHERE interface_device_id =?;");
			$query = parent::executeStatement($query, array($id));
		}
	}
	/*@NAME: deleteSwitchInterfaces
	 *@IN: $dbconnection un objet PDO
	*		$id id de l'interface
	*@OUT: /
	*@TREAT: supprime l'interface du switch de l'application
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function deleteSwitchInterface($dbconnection,$id)
	{
		if(is_numeric($id))
		{
			$query = $dbconnection->prepare("DELETE FROM esia_mac_address WHERE mac_interface_id =?;");
			$query = parent::executeStatement($query, array($id));
				
			$query = $dbconnection->prepare("DELETE FROM esia_error_happen WHERE happen_interface_id =?;");
			$query = parent::executeStatement($query, array($id));
				
			$query = $dbconnection->prepare("DELETE FROM esia_interface WHERE interface_id =?;");
			$query = parent::executeStatement($query, array($id));
		}
	}
	/*@NAME: updateSwitchInterfaceCritical
	 *@IN: $dbconnection l'objet de connexion PDO
	*		sw id du switch ou un objet switch
	*@OUT: le nombre d'interface du switch
	*@TREAT: retourne le nombre d'interface du switch selon la DB
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function countSwitchInterface($dbconnection,$sw)
	{
		$id=null;
		$query = $dbconnection->prepare("SELECT count(*) as nbr
				FROM esia_interface
				WHERE interface_device_id=:id");
		if(is_numeric($sw))
		{
			$id = $sw;
		}
		else
		{
			$id = $sw->getId();
		}
		$query = parent::executeStatement($query,array(':id' =>$id));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->nbr;
	}
	/*@NAME: searchMac
	 *@IN: $dbconnection l'objet de connexion PDO
	*		$mac la mac adresse ou une partie de la mac recherchée au format de base de donnée donc pas de ':','.' ou d'espace (exemple: 000bac679d60)
	*@OUT: un tableau associatif
	*	$arr[]['id'] id de l'interface
	*	$arr[]['name'] nom de l'interface
	*	$arr[]['plug'] prise ou est connectée l'interface
	*	$arr[]['description'] description de l'interface
	*	$arr[]['node_id'] id du noeud
	*	$arr[]['node_name'] nom du noeud
	*	$arr[]['node_description'] description du noeud
	*	$arr[]['mac_adresse'] adresse mac
	*	$arr[]['mac_time_deconnection'] heure de la deconnexion si égal à 0 la mac est connectée
	*	$arr[]['mac_vendor'] vendeur de la mac
	*@TREAT: retourne les mac-adresses correpondants à la recherche
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function searchMac($dbconnection,$mac)
	{
		$query = $dbconnection->prepare("SELECT interface_name,interface_plug,interface_desc,device_id,device_name,mac_address,mac_time_out_connection,device_description
				FROM esia_mac_address
				LEFT JOIN esia_interface ON interface_id = mac_interface_id
				LEFT JOIN esia_device ON interface_device_id = device_id
				WHERE mac_address LIKE ?");
		$qvendor = $dbconnection->prepare("SELECT vendor_company
				FROM esia_mac_address_vendor
				WHERE vendor_mac_address=?;");
		$query = parent::executeStatement($query,array("%".$mac."%"));
		$arr =  array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			preg_match("#^([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$#",$row->mac_address,$regs);
			$mac = $regs[1].":".$regs[2].":".$regs[3].":".$regs[4].":".$regs[5].":".$regs[6];
			//on recherche le vendeur de la mac adresse
			$qvendor = parent::executeStatement($qvendor,array($regs[1].$regs[2].$regs[3]));
			$rvendor=$qvendor->fetch(PDO::FETCH_OBJ);
			$arr[]= array('id'=>$row->interface_id,
					'name'=>$row->interface_name,
					'plug'=>$row->interface_plug,
					'description'=>$row->interface_desc,
					'node_id'=>$row->device_id,
					'node_name'=>$row->device_name,
					'node_description'=>$row->device_description,
					'mac_adresse'=>$row->mac_address,
					'mac_time_deconnection'=>$row->mac_time_out_connection,
					'mac_vendor'=>$rvendor->vendor_company);
		}
		return $arr;
	}
	/*@NAME: searchPlug
	 *@IN: $dbconnection l'objet de connexion PDO
	*		$plug la prise ou une partie du nom de la prise
	*@OUT: un tableau associatif
	*	$arr[]['id'] id de l'interface
	*	$arr[]['name'] nom de l'interface
	*	$arr[]['plug'] prise ou est connectée l'interface
	*	$arr[]['description'] description de l'interface
	*	$arr[]['node_id'] id du noeud
	*	$arr[]['node_name'] nom du noeud
	*	$arr[]['node_description'] description du noeud
	*@TREAT: retourne les interfaces qui sont connectés à la prise demandée
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function searchPlug($dbconnection,$plug)
	{
		$query = $dbconnection->prepare("SELECT interface_id,interface_name,interface_plug,interface_desc,device_id,device_name,device_description
				FROM esia_interface
				LEFT JOIN esia_device ON interface_device_id = device_id
				WHERE interface_plug ILIKE ?");
		$query = parent::executeStatement($query,array("%".$plug."%"));
		$arr =  array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]= array('id'=>$row->interface_id,
					'name'=>$row->interface_name,
					'plug'=>$row->interface_plug,
					'description'=>$row->interface_desc,
					'node_id'=>$row->device_id,
					'node_name'=>$row->device_name,
					'node_description'=>$row->device_description);
		}
		return $arr;
	}
	/*@NAME: searchPlug
	 *@IN: $dbconnection l'objet de connexion PDO
	*		$plug la prise ou une partie du nom de la prise
	*@OUT: un tableau associatif
	*	$arr[]['id'] id de l'interface
	*	$arr[]['name'] nom de l'interface
	*	$arr[]['plug'] prise ou est connectée l'interface
	*	$arr[]['description'] description de l'interface
	*	$arr[]['node_id'] id du noeud
	*	$arr[]['node_name'] nom du noeud
	*	$arr[]['node_description'] description du noeud
	*@TREAT: retourne les interfaces qui sont connectés à la prise demandée
	*@EXCEPTION: 	EsiaDbException si une erreur survient
	*/
	public function searchInterfaceDescription($dbconnection,$plug)
	{
		$query = $dbconnection->prepare("SELECT interface_id,interface_name,interface_plug,interface_desc,device_id,device_name,device_description
				FROM esia_interface
				LEFT JOIN esia_device ON interface_device_id = device_id
				WHERE interface_desc ILIKE ?");
		$query = parent::executeStatement($query,array("%".$plug."%"));
		$arr =  array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[]= array('id'=>$row->interface_id,
					'name'=>$row->interface_name,
					'plug'=>$row->interface_plug,
					'description'=>$row->interface_desc,
					'node_id'=>$row->device_id,
					'node_name'=>$row->device_name,
					'node_description'=>$row->device_description);
		}
		return $arr;
	}
	/**
	 * retourne les noeuds auquel le switch est connecté 
	 * ATTENTION ne test pas du tout les droit d'accès utilisateurs
	 * @param: $dbconnection l'objet de connexion PDO
	 * @param: $user_id identifiant de l'utilisateur
	 * @param: $nodeid identifiant du switch
	 * @return: une liste de noeuds connecté
	 * @exception: EsiaDbException si une erreur survient
	 */
	/**
	 * retourne les noeuds auquel le switch est connecté
	 * ATTENTION ne test pas du tout les droit d'accès utilisateurs
	 * @param: $dbconnection l'objet de connexion PDO
	 * @param: $user_id identifiant de l'utilisateur
	 * @param: $nodeid identifiant du switch
	 * @return: une liste de noeuds connecté
	 * @exception: EsiaDbException si une erreur survient
	 */
	static function getNodeConnected($dbconnection,$nodeid)
	{
		$query= $dbconnection->prepare("SELECT TSWINT.interface_id AS switch_interface_id,
											TSWINT.interface_name AS switch_interface,
											TSWINT.interface_ifindex AS switch_interface_ifindex,
											TNODE.device_id AS node_id,
											TNODE.device_name AS node_name,
											TNODEINT.interface_mac_addr AS node_mac,
											esia_interface_connected_to.conint_protocol,
                                            vendor_company
										FROM esia_device AS TSW
										JOIN esia_interface AS TSWINT ON TSW.device_id = TSWINT.interface_device_id
			
										LEFT JOIN esia_interface_connected_to ON conint_switch_interface_id = TSWINT.interface_id
										LEFT JOIN esia_interface AS TNODEINT ON TNODEINT.interface_id = conint_interface_id2
										LEFT JOIN esia_device AS TNODE ON TNODE.device_id = TNODEINT.interface_device_id
										LEFT JOIN esia_mac_address_vendor ON vendor_mac= trunc(TNODEINT.interface_mac_addr)
										WHERE TSW.device_id=:id
										ORDER BY TSWINT.interface_ifindex");
		$query = Model_esia::executeStatement($query,array(':id'=>$nodeid));
		$list=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$nodes=array();
			$nodes[] = array('id'=>$row->node_id,'name'=>$row->node_name ,'mac'=>$row->node_mac,'protocol' => $row->conint_protocol,'vendor'=>$row->vendor_company );
			if(array_key_exists($row->switch_interface_id, $list))
			{
				$list[$row->switch_interface_id]['nodes'] = array_merge($list[$row->switch_interface_id]['nodes'],$nodes);
			}
			else
			{
				$list[$row->switch_interface_id]= array('interface_id'=>$row->switch_interface_id,
					'interface_name'=>$row->switch_interface,
					'interface_ifindex'=>$row->switch_interface_ifindex,
					'nodes'=> $nodes
				);
				
			}
		}
		return $list;
	}
	static function getNodeMACConnected($dbconnection,$nodeid)
	{
		$query= $dbconnection->prepare("SELECT connected_id,connected_description, connected_mac, connected_protocol, connected_interface_id,interface_ifindex,vendor_company
                                        FROM esia_interface_node_connected
                                        JOIN esia_interface ON interface_id =connected_interface_id
                                        LEFT JOIN esia_mac_address_vendor ON vendor_mac= trunc(connected_mac)
                                        WHERE interface_device_id = ?;");
		$query = Model_esia::executeStatement($query,array($nodeid));
		$list=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[$row->connected_interface_id][] = array('id'=>$row->connected_id,'description'=>$row->connected_description ,'mac'=>$row->connected_mac,'protocol' => $row->connected_protocol,
				'vendor'=>$row->vendor_company);
			/*if(array_key_exists($row->connected_interface_id, $list))
			 {
			 
			 }
			 else
			 {
			 $list[$row->switch_interface_id]= array('interface_id'=>$row->switch_interface_id,
			 'interface_name'=>$row->switch_interface,
			 'interface_ifindex'=>$row->switch_interface_ifindex,
			 'nodes'=> $nodes
			 );
			 
			 }*/
		}
		return $list;
	}
	static function askScan($dbconnection,$nodeid)
	{
		$query= $dbconnection->prepare("SELECT * FROM esia_switch_scan WHERE switch_scan_id=?");
		$query = Model_esia::executeStatement($query,array($nodeid));
		$arr = $query->fetchAll();
		if(count($arr) == 0)
		{
			$query= $dbconnection->prepare("INSERT INTO esia_switch_scan(switch_scan_id, switch_scan_time, switch_scan_state) VALUES (?,NOW(),'START');");
			$query = Model_esia::executeStatement($query,array($nodeid));
		}
		else
		{
			$query= $dbconnection->prepare("UPDATE esia_switch_scan SET switch_scan_time=NOW(),switch_scan_state='START' WHERE switch_scan_id=? ");
			$query = Model_esia::executeStatement($query,array($nodeid));
		}
		
	}
	static function getScanState($dbconnection,$nodeid=null,$state=null)
	{
		$sql = "SELECT switch_scan_id as nodeid,switch_scan_state as state,to_char(switch_scan_time,'DD-MM-YYYY HH24:MI:SS') as lasttime
                FROM esia_switch_scan ";
		if(!empty($nodeid))
		{
			$query= $dbconnection->prepare($sql."WHERE switch_scan_id=?");
			$query = Model_esia::executeStatement($query,array($nodeid));
			$arr = $query->fetchAll();
			return $arr[0];
		}
		if(!empty($state))
		{
			$query= $dbconnection->prepare($sql."WHERE switch_scan_state=?");
			$query = Model_esia::executeStatement($query,array($state));
			$arr = $query->fetchAll();
			return $arr;
		}
		
	}
	static function UpdateScanState($dbconnection,$nodeid,$state)
	{
		$query= $dbconnection->prepare("UPDATE esia_switch_scan SET switch_scan_time=NOW(),switch_scan_state=? WHERE switch_scan_id=? ");
		$query = Model_esia::executeStatement($query,array($state,$nodeid));
	}
}
class ModelSwitchException extends EsiaDbException{}

?>
